//CHANGED FOR POSTGRES
/************************************
 * dbConn
 * @author Matthew Teeter, based off of work by Razvan Petrescu
 * Java database connection manager
 * Works with Postgress via JDBC
 ************************************/


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.ArrayList;

public class dbConn {
	private static int MaxConnection = 0;
	private int HConnection;
	private String IPAddr;
	private String Login;
	private String Password;
	private String Database;
	private String Error = "";
	private Connection conn = null;
	public boolean connected;
	private ArrayList<String> queries = new ArrayList<String>();

	/********************************
	 * Constructor
	 ********************************/
	public dbConn( String pIPAddr, String pLogin, String pPassword, String pDatabase )
	{
		IPAddr = pIPAddr;
		Login = pLogin;
		Password = pPassword;
		Database = pDatabase;

		try
		{
			Class.forName( "org.postgresql.Driver");
		}
		catch( Exception x )
		{
			System.out.println("Please make sure the PostgreSQL driver is in your CLASSPATH.");
			Error = x.toString();
			HConnection = -1;
			return;
		}

		if( Success())
		{
			dbConn.MaxConnection ++;
			HConnection = dbConn.MaxConnection;
		}
		else
		{
			HConnection = -1;
			System.out.println("\t* Connect failed: " + Error);
		}
	}

	/********************************
	 * Connection handle accessor
	 ********************************/
	public int getConnection(){
		return HConnection;
	}


	/********************************
	 * Validate connection info
	 ********************************/
	public boolean Success(){
		if( Connect()){
			connected=true;//Disconnect();//switch out
			return true;
		}else{
			connected=false;
			Error += " Connect unsuccessful.";
			return false;
		}
	}


	/********************************
	 * Connect to database
	 ********************************/
	public boolean Connect(){
		try{
			//use the default port for postgres, 5432
			String sConn = "jdbc:postgresql://" + IPAddr + ":5432/" + Database + "";
			System.out.println( "\t* Attempting to connect to: " + sConn + "(" + Login + ", " + Password + ")");
			conn = DriverManager.getConnection( sConn, Login, Password );
		}catch( Exception x ){
			connected=false;
			Error = x.toString();
			return false;
		}
		Error = "";
		return true;
	}


	/********************************
	 * Disconnect from database
	 ********************************/
	public void Disconnect(){
		try{
			conn.close();
			connected=false;
		}catch( Exception x){
			System.out.println("\t* Error closing connection: " + x.toString());
		}
	}


	/***********************************
	 * Execute command against database
	 ***********************************/
	public String Execute( String command ){
		//store this query
		//queries.add(command);
		//We deal with one query at a time so just return the string!
		
		String Return = "";
		String TypeOfReturn = "Return";//Return>
		int counter = 1;
		if (!connected)
		{
			System.out.println("Cant send that query, we are not connected to the database.");
			return "Error";
		}
		try{
			//Connect();//connects each time... wasteful but it handles the closing problem.
					//if it is too slow we can code around it and send pings every so often
			Statement dbStatement = conn.createStatement( ResultSet.TYPE_SCROLL_INSENSITIVE,
				ResultSet.CONCUR_READ_ONLY );

			ResultSet dbResults = dbStatement.executeQuery( command );
			ResultSetMetaData rsmd = dbResults.getMetaData();
	        int numColumns = rsmd.getColumnCount();
	        // Get the column names; column indices start from 1
			Return += " query=\"" + replaceEntities(command) + "\">";
			//Return += " numReturned='" + dbResults.getFetchSize() + "'";//check this
			//Return += ">";
			while( dbResults.next())//this does for multiple results... problem if multiple rows returned; they will simply be appended
			{
				//Return += " attr" + counter + "='" + dbResults.getString( 1 ) + "'";
				Return += " <" + counter ;//+ ">"; 
				//deal with multiple columns
				for (int i=1; i<numColumns+1; i++) 
				{
					if (rsmd.getColumnName(i).equals("title")|| rsmd.getColumnName(i).equals("filename") || rsmd.getColumnName(i).equals("subtitle"))
					{
						if (dbResults.getString(i).contains("\""))
						{
							Return += " " + rsmd.getColumnName(i) + "='" + dbResults.getString( i ) + "'";
						}
						else
						{
							Return += " " + rsmd.getColumnName(i) + "=\"" + dbResults.getString( i ) + "\"";
						}
					}
					else
					{
						Return += " " + rsmd.getColumnName(i) + "='" + dbResults.getString( i ) + "'";
					}
				}
				Return += "></" + counter++ + ">"; 
			}
		}catch( Exception x ){
			Return = " errortype='" + x.toString() + "'>";
			Error = x.toString();
			TypeOfReturn = "Error";
		}
		return "<" + TypeOfReturn  + Return + "</" + TypeOfReturn + ">\0";
			// '\0' required by Flash
	}

	//Parse out < > and replace
	public String replaceEntities(String s)
	{
		s = s.replaceAll("<", "&lt;");
		s = s.replaceAll(">", "&gt;");
		return s;
	}

	/********************************
	 * Error accessor
	 ********************************/
	public String getError(){
		return Error;
	}
}
